from pydantic import Field

from definition_tooling.converter import CamelCaseModel, DataProductDefinition


class CurrentWeatherMetricRequest(CamelCaseModel):
    lat: float = Field(
        ...,
        title="Latitude",
        description="The latitude coordinate of the desired location",
        ge=-90.0,
        le=90.0,
        examples=[60.192059],
    )
    lon: float = Field(
        ...,
        title="Longitude",
        description="The longitude coordinate of the desired location",
        ge=-180.0,
        le=180.0,
        examples=[24.945831],
    )


class CurrentWeatherMetricResponse(CamelCaseModel):
    humidity: float = Field(
        ..., title="Current relative air humidity in %", examples=[72]
    )
    pressure: float = Field(..., title="Current air pressure in hPa", examples=[1007])
    rain: bool = Field(
        ..., title="Rain status", description="If it's currently raining or not."
    )
    temp: float = Field(
        ..., title="Current temperature in Celsius", examples=[17.3], ge=-273.15
    )
    wind_speed: float = Field(
        ..., title="Current wind speed in m/s", examples=[2.1], ge=0
    )
    wind_direction: float = Field(
        ...,
        title="Current wind direction in meteorological wind direction degrees",
        ge=0,
        le=360,
        examples=[220.0],
    )


DEFINITION = DataProductDefinition(
    version="1.0.0",
    title="Current weather in a given location",
    description="Current weather in a given location in metric units",
    request=CurrentWeatherMetricRequest,
    response=CurrentWeatherMetricResponse,
    requires_authorization=True,
    requires_consent=True,
)



from pydantic import Field

from definition_tooling.converter import (
    CamelCaseModel,
    DataProductDefinition,
    ErrorResponse,
)


class CoffeeBrewingRequest(CamelCaseModel):
    brew: str = Field(
        ...,
        title="Brew",
        description="Kind of drink to brew",
        examples=["coffee"],
    )


class CoffeeBrewingResponse(CamelCaseModel):
    ok: bool = Field(
        ...,
        title="OK",
        examples=[True],
    )


@ErrorResponse(description="I'm a teapot")
class TeaPotError(CamelCaseModel):
    ok: bool = Field(
        ...,
        title="OK",
        examples=[False],
    )
    error_message: str = Field(
        ...,
        title="Error message",
        examples=["I'm a teapot"],
    )


DEFINITION = DataProductDefinition(
    version="0.1.0",
    title="Coffee brewer",
    description="Coffee brewer",
    request=CoffeeBrewingRequest,
    response=CoffeeBrewingResponse,
    error_responses={
        418: TeaPotError,
    },
    deprecated=True,
)



import pytest
from semver import Version

from definition_tooling.validator.utils import parse_version_without_patch


@pytest.mark.parametrize(
    "version_string,expected",
    [
        ["0.0", "0.0.0"],
        ["0.1", "0.1.0"],
        ["0.1-beta", "0.1.0-beta"],
        ["0.1-beta.2", "0.1.0-beta.2"],
        ["0.1-beta.2+build", "0.1.0-beta.2+build"],
    ],
)
def test_parse_version_without_patch(version_string: str, expected: str):
    assert parse_version_without_patch(version_string) == Version.parse(expected)


@pytest.mark.parametrize(
    "version_string",
    [
        "",
        "0",
        "1",
        "a.b",
        "1.1.1",
    ],
)
def test_invalid_parse_version_without_patch(version_string):
    with pytest.raises(ValueError):
        parse_version_without_patch(version_string)
